@@ -273,6 +273,8 @@ static void _hmac(unsigned char *md, void *key, int key_len, char *data) {
 	ctx = HMAC_CTX_new();
 #else
 	ctx = &_ctx;
+	/* work-around crash in certain versions of libssl */
+	HMAC_CTX_init(ctx);
 #endif
 	HMAC_Init_ex(ctx, key, key_len, EVP_sha256(), NULL);
 	HMAC_Update(ctx, (unsigned char*)data, strlen(data));
@@ -294,9 +296,11 @@ static int _curl_trace(CURL *handle, curl_infotype type,
 
 	switch (type) {
 	case CURLINFO_TEXT:
-	fprintf(stderr, "== Info: %s", data);
+		fprintf(stderr, "== Info: %s", data);
+		/* fall through */
 	default:
 	case CURLINFO_SSL_DATA_OUT:
+		/* fall through */
 	case CURLINFO_SSL_DATA_IN:
 		return 0;
 
